/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "MidgardBasicElement.hh"
#include "Typen.hh"
#include "Grundwerte.hh"
#include "Fertigkeiten.hh"
#include "Waffe.hh"
#include "WaffeGrund.hh"
#include "KiDo.hh"
#include "Zauber.hh"
#include "Zauberwerk.hh"
#include "Sprache.hh"
#include "Schrift.hh"
//#include "class_SimpleTree.hh"
//#include "SimpleTree.hh"
#include <Misc/itos.h>
//#include "xml.h"
#include "Datenbank.hh"
#include "Abenteurer.hh"
#include <Misc/germanstring.h>
#include <memory>
#include "NotFound.h"
#include "Ausgabe.hh"
#include "LernListen.hh"
#include <stdio.h> // for snprintf

#ifdef _MSC_VER
#define snprintf _snprintf
#endif

bool MBEm::do_not_timestamp;

void MidgardBasicElement_mutable::addErfolgswert(int e, wie_gesteigert const& w)
{
  gesteigert.push_back(w);
  gesteigert.back().alter_wert=erfolgswert;
  if (!do_not_timestamp)
	  gesteigert.back().wann=time(0);
  erfolgswert+=e;
}

void MidgardBasicElement_mutable::setLernArt(wie_erlernt const& w)
{	gelernt=w;
	if (!do_not_timestamp)
		gelernt.wann=time(0);
}

bool LernSchemaElement::sort::operator() (Handle<LernSchemaElement const> const& x,Handle<LernSchemaElement const> const& y) const
{ switch(es) {
               case(LERNPUNKTEPFLICHT) : return x->Pflicht() > y->Pflicht() ||
                  (x->Pflicht() == y->Pflicht()  &&  x->Lernpunkte() < y->Lernpunkte() ) ;
               case(NAME) : return germanstring(x->getMBE()->Name()) < germanstring(y->getMBE()->Name())  ;
               case(ERFOLGSWERT): return x->Erfolgswert() > y->Erfolgswert();
           }abort();
}

bool H_MBEm::sort::operator() (H_MBEm x,H_MBEm y) const
{ switch(es) {
               case(NAME) : return germanstring(x->getMBE()->Name()) < germanstring(y->getMBE()->Name())  ;
               case(ERFOLGSWERT): return x->Erfolgswert() > y->Erfolgswert();
           }abort();
}

bool cH_MBE::sort::operator() (cH_MBE const& x,cH_MBE const& y) const
{ switch(es) {
               case(NAME) : return germanstring(x->Name()) < germanstring(y->Name())  ;
               case(ERFOLGSWERT): return x->Anfangswert() > y->Anfangswert();
           }abort();
}

bool MidgardBasicElement::Voraussetzung(const Abenteurer& A,bool anzeigen) const
{Ausgabe(Ausgabe::Error,"ERROR in Voraussetzung");return false;}

std::string MidgardBasicElement::RegionString() const
{
  std::vector<cH_Region> V=Datenbank.Regionen;
  std::string s=Regionen_All::getRegionfromAbk(V,region)->Name();
  if(!RegionZusatz().empty()) s+=" ("+RegionZusatz()+")";
  return s;
}

bool MidgardBasicElement::ist_lernbar(const std::vector<cH_Typen>& Typ,const std::map<std::string,std::string>& map_typ) const
{
  for (std::vector<cH_Typen>::const_iterator i=Typ.begin();i!=Typ.end();++i)
    if (!const_cast<std::map<std::string,std::string>& >(map_typ)[(*i)->Short()].empty())
      return true;
  return false;
}

bool MidgardBasicElement_mutable::ist_gelernt(const std::list<H_MBEm>& L) const
{
 for (std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
     if(**i==*this) return true;
 return false;
}

bool LernSchemaElement::ist_gelernt(Abenteurer const& A) const
{
  if (!this->lernpunkte) // genauer hinsehen
  {
    if (getMBE()->Name()=="Muttersprache") return true; // was selected earlier
    if (getMBE()->Name()=="Gastlandsprache") return A.List_Sprache().size()>1;
  }
  if (getMBE()->What()==MBE_T::FERTIGKEIT && ist_gelernt(A.List_Fertigkeit())) return true;
  if (getMBE()->What()==MBE_T::WAFFE && ist_gelernt(A.List_Waffen())) return true;
  return false;
}

bool LernSchemaElement::ist_gelernt(const std::list<H_LSE>& L) const
{
 for (std::list<H_LSE>::const_iterator i=L.begin();i!=L.end();++i)
     if(**i==*this) return true;
 return false;
}

// is this really needed?
bool LernSchemaElement::ist_gelernt(const std::list<H_MBEm>& L) const
{
 for (std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
     if(**i==*this) return true;
 return false;
}

#if 0
bool MidgardBasicElement::ist_gelernt(const std::list<std::string>& L) const
{
 for (std::list<std::string>::const_iterator i=L.begin();i!=L.end();++i)
   {
//cout <<"Ist gelernt? " << (*i)<<'\t'<<Name()<<'\n';
     if((*i)==Name()) return true;
     if(Name()=="Muttersprache" &&
         (*i).find("_Muttersprache")!=std::string::npos) return true;
     if(Name()=="Gastlandsprache" &&
         (*i).find("_Gastlandsprache")!=std::string::npos) return true;
   }
 return false;
}
#endif

std::string MidgardBasicElement::Standard__(const Abenteurer &A) const
{
 std::vector<std::string> s = Standard(A);
 std::string s2=s[0];
 if (!A.Typ1()->Short().empty() && !A.Typ2()->Short().empty()) s2+='/';
 if (!A.Typ2()->Short().empty()) s2+=s[1];
 return s2;
}

std::vector<std::string> MidgardBasicElement::Standard(const Abenteurer &A) const
{
 assert(A.getVTyp().size()==2);
 std::vector<std::string> s(2);
 for(std::map<std::string,std::string>::const_iterator i=map_typ.begin();i!=map_typ.end();++i)
   if(A.Typ1()->Short()==i->first) {s[0]=i->second; break;}
 for(std::map<std::string,std::string>::const_iterator i=map_typ.begin();i!=map_typ.end();++i)
   if(A.Typ2()->Short()==i->first) {s[1]=i->second; break;}

 s[0]=AusnahmenString(A,A.Typ1(),s[0]);
 s[1]=AusnahmenString(A,A.Typ2(),s[1]);

 return s;
}

bool MidgardBasicElement::Grundfertigkeit(const Abenteurer &A) const
{
  if      (standard_one_G(Standard(A)) ) return true;
  return false;
}

bool MidgardBasicElement::Standardfertigkeit(const Abenteurer &A) const
{
  if (standard_all_S(Standard(A)) ) return true;
  return false;
}


std::string MidgardBasicElement::AusnahmenString(const Grundwerte &Werte,const cH_Typen& Typ,const std::string s) const
{
  if(Region().empty()) return s;
  for(std::vector<st_ausnahmen>::const_iterator i=VAusnahmen.begin();i!=VAusnahmen.end();++i)
   {
     if( (i->herkunft.empty()|| i->herkunft==Werte.Herkunft()->Name()) &&
         (i->spezies .empty()|| i->spezies ==Werte.Spezies()->Name())  &&
         (i->stand   .empty()|| i->stand==Werte.Stand())  &&
         (i->typ     .empty()|| i->typ==Typ->Short() ))
      {
       return i->standard;
      }
   }
  return s;
}


double MidgardBasicElement::Standard_Faktor(const Abenteurer &A) const
{
  double fac;
  if      (standard_one_G(Standard(A)) ) fac = 0.5;
  else if (standard_all_S(Standard(A)) ) fac = 1.0;
  else
    {
      fac = 2.0;
      if (What()==ZAUBER || What()==ZAUBERWERK) fac=5.0;
    }
//cout <<Name()<<' ' <<"StandardFaktor = "<<fac<<"  Grundkosten= "<<GrundKosten()<<'\n';
  return fac;
}

bool MidgardBasicElement::standard_one_G(const std::vector<std::string>& s) const
{
 assert(s.size()==2);
 if (s[0] == "G" || s[1] =="G" ) return true;
 return false;
}

bool MidgardBasicElement::standard_all_S(const std::vector<std::string>& s) const
{
 assert(s.size()==2);
 if (s[0] == "S" && (s[1] =="S" || s[1].empty())) return true;
 if (s[1] == "S" && (s[0] =="S" || s[0].empty())) return true;
 return false;
}


int MidgardBasicElement::get_Steigern_Kosten(int erfolgswert) const
{
 return const_cast<std::map<int,int>& >(map_erfolgswert_kosten)[erfolgswert];
}

int MidgardBasicElement_mutable::Steigern(const Abenteurer &A) const
{
   int kosten=0;
   if(Erfolgswert()>0)
      kosten = (*this)->get_Steigern_Kosten(Erfolgswert()+1);
   else
      kosten = (*this)->get_Steigern_Kosten(-Erfolgswert()-1);
   int back = int((*this)->Standard_Faktor(A)*kosten);
   return back;
}

// Anzahl der FP für eine Reduktion des Erfolgswertes (Funkt. sollte nicht nötig sein)
int MidgardBasicElement_mutable::Reduzieren(const Abenteurer &A) const
{
   int kosten=0;
   if(Erfolgswert()>0)
      kosten = (*this)->get_Steigern_Kosten(Erfolgswert());
   else
      kosten = (*this)->get_Steigern_Kosten(-Erfolgswert());
   int back = int((*this)->Standard_Faktor(A)*kosten);
   return back;
}

// Anzahl der FP für ein Verlernen der Fähigkeit (Funkt. sollte nicht nötig sein)
int MidgardBasicElement_mutable::Verlernen(const Abenteurer &A) const
{
   if(Reduzieren(A)==0)
    {
        return (*this)->Kosten(A);
    }
   else return 0;
}


void MidgardBasicElement::get_map_typ(const Tag &t)
{ static std::string Grund("Grund"), Standard("Standard"), Ausnahme("Ausnahme");
  static std::string Typ("Typ"),G("G"),S("S"),A("A");
   FOR_EACH_CONST_TAG(i,t)
    {  if (i->Type()==Grund) map_typ[i->getAttr(Typ)]=G;
       else if (i->Type()==Standard) map_typ[i->getAttr(Typ)]=S;
       else if (i->Type()==Ausnahme) map_typ[i->getAttr(Typ)]=A;
    }
}

void MidgardBasicElement::get_map_typ()
{
 static std::auto_ptr<cH_Fertigkeit> sprache,schrift,kido;

 if(What()==SPRACHE)
 {  if (!sprache.get()) sprache.reset(new cH_Fertigkeit("Sprache"));
    map_typ=(*sprache)->map_typ;
 }
 else if(What()==SCHRIFT)
 {  if (!schrift.get()) schrift.reset(new cH_Fertigkeit("Schreiben"));
    map_typ=(*schrift)->map_typ;
 }
 else if(What()==KIDO)
 {  if (!kido.get()) kido.reset(new cH_Fertigkeit("KiDo"));
    map_typ=(*kido)->map_typ;
 }
 else assert(!"get_map_typ mit merkwürdigem Typ");
}


const std::map<int,int> &MidgardBasicElement::get_Steigern_Kosten_map()
{
 if(What()==WAFFE)
 {  int schwierigkeit=dynamic_cast<Waffe&>(*this).Schwierigkeit();
    map_erfolgswert_kosten=waffen_steigern_nach_schwierigkeit[schwierigkeit];
 }
 else if(What()==SPRACHE)
 { map_erfolgswert_kosten=cH_Fertigkeit("Sprache")->map_erfolgswert_kosten; }
 else if(What()==SCHRIFT)
 { map_erfolgswert_kosten=cH_Fertigkeit("Schreiben")->map_erfolgswert_kosten; }
 else assert(!"get here");

 return map_erfolgswert_kosten;
}

void MidgardBasicElement::load_steigern_kosten(const Tag &t)
{   std::map<int,int> &m=sonstige_steigern_kosten[t.getAttr("Fertigkeit")];
    for (int i=1;i<=22;++i)
       m[i]=t.getIntAttr("Wert"+itos(i),0);
}

void MidgardBasicElement::load_waffen_steigern_nach_schwierigkeit(const Tag &t)
{   std::map<int,int> &m=waffen_steigern_nach_schwierigkeit[t.getIntAttr("Schwierigkeit")];
    for (int i=1;i<=22;++i)
       m[i]=t.getIntAttr("Wert"+itos(i),0);
}

void MidgardBasicElement::get_Steigern_Kosten_map(const Tag &t)
{const Tag *kosten=t.find("Kosten");

 if (!kosten)
 {  // steigern_wie herausfinden
    std::string steigern_wie;
    const Tag *steigern_wie_t=t.find("steigern_wie");
    if (steigern_wie_t) steigern_wie=steigern_wie_t->getAttr("Fertigkeit");
    else
    {  Ausgabe(Ausgabe::Warning,"keine Kosten für '"+Name()+"' gefunden");
       return;
    }

    // Kosten für steigern_wie suchen:
    try
    {  if (steigern_wie=="-" || steigern_wie.empty()) return;
       else
       {  std::map<std::string,std::map<int,int> >::const_iterator i=
       		sonstige_steigern_kosten.find(steigern_wie);
       	  if (i!=sonstige_steigern_kosten.end())
       	  {  map_erfolgswert_kosten=i->second;
       	     return;
       	  }

          map_erfolgswert_kosten=cH_Fertigkeit(steigern_wie)->map_erfolgswert_kosten;
          return;
       }
    }
    catch (const NotFound &e) // keine Fertigkeit
    {  Ausgabe(Ausgabe::Warning,"keine Kosten für '"+Name()+"/"+steigern_wie+"' gefunden");
       return;
    }
 }
 else
    for (int i=1;i<=22;++i)
       map_erfolgswert_kosten[i]=kosten->getIntAttr("Wert"+itos(i),0);
}

static void fp2xml(Tag &t, MBEm::wie_erlernt::fp_t const& fp)
{
	if (fp.aep) t.setAttr("AEP",fp.aep);
	if (fp.kep) t.setAttr("KEP",fp.kep);
	if (fp.zep) t.setAttr("ZEP",fp.zep);
}

static void xml2fp(Tag const&t, MBEm::wie_erlernt::fp_t & fp)
{
	fp.aep=t.getAttr_def<int>("AEP");
	fp.kep=t.getAttr_def<int>("KEP");
	fp.zep=t.getAttr_def<int>("ZEP");
}

static std::string time2xml(time_t t)
{
	struct tm *tm;
	char buf[30];
	tm=localtime(&t);
	snprintf(buf, sizeof buf, "%04d-%02d-%02dT%02d:%02d:%02d+%d", tm->tm_year+1900,
		      tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min,
		      tm->tm_sec, tm->tm_isdst ? 2 : 1); // hardcoded to germany :-(
	return buf;
}

static time_t xml2time(std::string const& s)
{
	struct tm tm;
	int gmtoff=0;
	if (s.empty()) return 0;
	memset(&tm,0,sizeof tm);
	sscanf(s.c_str(), "%d-%d-%dT%d:%d:%d+%d", &tm.tm_year,
		      &tm.tm_mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min,
		      &tm.tm_sec, &gmtoff);
	--tm.tm_mon;
	tm.tm_year-=1900;
	tm.tm_isdst = gmtoff==2;
	return mktime(&tm);
}

static std::string liste2xml(MBEm::wie_erlernt::lernen_t l)
{
	if (l==MBEm::wie_erlernt::ungewoehnlich) return "ungewöhnlich";
	if (l==MBEm::wie_erlernt::fachkenntnis) return "Fachkenntnis";
	if (l==MBEm::wie_erlernt::allgemeinwissen) return "Allgemeinwissen";
	if (l==MBEm::wie_erlernt::separat) return std::string();
	return "<unknown>";
}

static MBEm::wie_erlernt::lernen_t xml2liste(std::string const& l)
{
	if (l=="ungewöhnlich") return MBEm::wie_erlernt::ungewoehnlich;
	if (l=="Fachkenntnis") return MBEm::wie_erlernt::fachkenntnis;
	if (l=="Allgemeinwissen") return MBEm::wie_erlernt::allgemeinwissen;
	return MBEm::wie_erlernt::separat;
}

void MidgardBasicElement::saveElementliste(Tag &datei,
			   const std::list<H_MBEm>& b,
                           const Grundwerte& Werte,
                           const std::vector<cH_Typen>& Typ)
{
  if(b.size()==0) return;
  for (std::list<H_MBEm>::const_iterator i=b.begin();i!=b.end();++i)
   {
      // oder <Beruf Wert=12>Arzt</Beruf> ?
      std::string type=(*(*i))->What_str();
      if (type.find('.')!=std::string::npos)
         type.replace(type.find('.'),1,"-");
      Tag &t=datei.push_back(Tag(type));
      t.setAttr("Bezeichnung", (*(*i))->Name());
      t.setIntAttr_nn("Wert", (*i)->Erfolgswert());
      t.setIntAttr_nn("Praxispunkte", (*i)->Praxispunkte());

      if ((*(*i))->What()==ZAUBERWERK)
      {
        t.setAttr("Art", cH_Zauberwerk((*i)->getMBE())->Art());
        t.setAttr("Stufe", cH_Zauberwerk((*i)->getMBE())->Stufe());
      }
      if ((*(*i))->What()==SCHRIFT)
      { t.setAttr("Buchstaben", cH_Schrift((*i)->getMBE())->Art_der_Schrift());
//        t.setAttr("Sprache","???");
      }
      if ((*(*i))->ZusatzEnum(Typ))
         t.setAttr("Zusatz", (*i)->Zusatz());
      if ((*i)->gelernt.wann)
      {  Tag &e=t.push_back(Tag("erlernt"));
      	 if ((*i)->gelernt.wann)
      	 	e.setAttr("wann",time2xml((*i)->gelernt.wann));
      	 switch((*i)->gelernt.wie)
      	 {  case MBEm::wie_erlernt::angeboren:
      		    e.setAttr("angeboren",true);
      	 		break;
      	 	case MBEm::wie_erlernt::beruf:
      	      	e.setAttr("Beruf",true);
      	      	break;
      	    case MBEm::wie_erlernt::lernen:
      	        e.setAttr("LP",(*i)->gelernt.lernen_detail.lp);
      	        if ((*i)->gelernt.lernen_detail.liste!=MBEm_T::wie_erlernt::separat)
      	        	e.setAttr("Liste",liste2xml((*i)->gelernt.lernen_detail.liste));
      	      	break;
     	    case MBEm::wie_erlernt::spruchrolle:
      	        e.setAttr("Spruchrolle",true);
     	    	fp2xml(e,(*i)->gelernt.spruchrolle_detail);
     	        break;
      	 	case MBEm::wie_erlernt::praxis:
      	 		fp2xml(e,(*i)->gelernt.praxis_detail.fp);
      	        e.setAttr<int>("PP",(*i)->gelernt.praxis_detail.praxispunkte);
      	      	break;
      	 	case MBEm::wie_erlernt::unterweisung:
      	 	case MBEm::wie_erlernt::selbststudium:
      	 		fp2xml(e,(*i)->gelernt.unterweisung_detail.fp);
      	        e.setAttr<int>("GS",(*i)->gelernt.unterweisung_detail.gs);
      	      	break;
      	 }
      }
      // wie gesteigert
      for (std::vector<MBEm::wie_gesteigert>::const_iterator j
    		  	=(*i)->gesteigert.begin(); j!=(*i)->gesteigert.end(); ++j)
      {
    	  Tag &g = t.push_back(Tag("gesteigert"));
    	  if (j->wann) g.setAttr("wann",time2xml(j->wann));
    	  g.setAttr("von",j->alter_wert);
       	 switch(j->wie)
       	 {  case MBEm::wie_erlernt::beruf:
       	      	g.setAttr("Beruf",true);
       	      	break;
       	    case MBEm::wie_erlernt::praxis:
       	 		fp2xml(g,j->praxis_detail.fp);
       	        g.setAttr<int>("PP",j->praxis_detail.praxispunkte);
       	      	break;
       	 	case MBEm::wie_erlernt::unterweisung:
       	 	case MBEm::wie_erlernt::selbststudium:
       	 		fp2xml(g,j->unterweisung_detail.fp);
       	        if (j->unterweisung_detail.gs)
       	        	g.setAttr<int>("GS",j->unterweisung_detail.gs);
       	      	break;
       	 }
      }
   }
}

  //////////////////////////////////////////////////////////////////////
  //Steigern von Fertigkeiten

std::map<std::string,MBE_T::EP_t> MBE_T::verwendbareEP;

void MidgardBasicElement::EP_steigern(const std::string &fert,EP_t e)
{  verwendbareEP[fert]=e;
}

MBE_T::EP_t MBE_T::EP_steigern(const std::string &fert)
{  std::map<std::string,EP_t>::const_iterator i=verwendbareEP.find(fert);
   if (i!=verwendbareEP.end()) { steigern_mit_EP=i->second; return steigern_mit_EP; }
   try
   {  cH_Fertigkeit f=cH_Fertigkeit(fert);
      steigern_mit_EP=f->Steigern_mit_EP();
      return steigern_mit_EP;
   } catch (const NotFound &f)
   {  Ausgabe(Ausgabe::Warning, "EP_steigern: Fertigkeit "+fert+" (noch) unbekannt");
      return Nicht;
   }
}

MBE_T::EP_t MBE_T::EP_steigern_load(const Tag &t)
{ EP_t res=Nicht;
  const Tag *steigern=t.find("EP-Typ");
  if (steigern)
  {  if (steigern->getBoolAttr("KEP")) res|=KEP;
     if (steigern->getBoolAttr("ZEP")) res|=ZEP;
  }
  return res;
}

std::string LernSchemaElement::Pflicht_str() const
{
  if (Pflicht()) return "X";
  else return "";
}

int MBE_T::FErfolgswert(const Abenteurer &abenteurer,const H_MBEm &mbem) const
{  return 0;
}

int MBE_T::Kosten(const Abenteurer &A) const
{return (int)(Standard_Faktor(A)*GrundKosten());}

void operator|=(MBE_T::EP_t &a, MBE_T::EP_t b)
{  (int&)a|=int(b);
}

bool operator&(MBE_T::EP_t a, MBE_T::EP_t b)
{  return int(a)&int(b);
}

std::map<int,std::map<int,int> > MidgardBasicElement::waffen_steigern_nach_schwierigkeit;
std::map<std::string,std::map<int,int> > MidgardBasicElement::sonstige_steigern_kosten;

H_MidgardBasicElement_mutable H_MidgardBasicElement_mutable::dup() const
{ return H_MidgardBasicElement_mutable((*this)->dup());
}

MidgardBasicElement_mutable *MidgardBasicElement_mutable::dup() const
{ MBEm *result=new MBEm(mbe);
  result->praxispunkte=praxispunkte;
  result->erfolgswert=erfolgswert;
  result->zusatz=zusatz;
  result->gelernt=gelernt;
  result->gesteigert=gesteigert;
  return result;
}

H_MBEm LSE_T::lernen(cH_MBE const& mbe, MBEm::wie_erlernt const&w, LSE_T::lernen_param const& param)
{  	H_MBEm result(mbe);
	if(mbe->Name()=="Landeskunde (Heimat)")
	{
		result=H_MBEm(cH_Fertigkeit("Landeskunde"));
		result->setZusatz(param.Aben.Herkunft()->Name());
	}
	else if(mbe->What()==MBE_T::FERTIGKEIT)
      {
		 // cH_Fertigkeit(getMBE())->get_region_lp(lp,Aben); // was tut das? holte Lernpunktanzahl
         if (mbe->Name()=="Muttersprache")
         {
            result->setErfolgswert(Sprache::ErfolgswertMuttersprache(param.Aben.In(),cH_Fertigkeit(mbe)->AttributBonus(param.Aben)));
         }
         else if(mbe->Name()=="Gastlandsprache")
         {
            result->setErfolgswert(Sprache::ErfolgswertGastlandsprache(param.Aben.In()));
         }
         goto zusaetze_abfragen;
      }

	// davon gibt es doch bestimmt noch viel mehr ...
	else
	{
	zusaetze_abfragen:
		MBE_T::eZusatz was=mbe->ZusatzEnum(param.Aben.getVTyp());
		if(was)
		{
			MBE_T::st_zusatz zus=Auswaehlen(mbe, param);
			// Sprache und Schrift erfordern ganz neues Objekt
			if (zus.name.empty() && !zus.mbe && !zus.land)
			{
			  Ausgabe(Ausgabe::Error, "Keine Variante von "+mbe->Name()+" erlernbar");
			  return H_MBEm((MidgardBasicElement_mutable*)0);
			}
			if(was==MBE_T::ZSprache || was==MBE_T::ZSchrift)
			{
				H_MBEm neu_obj(zus.mbe);
				neu_obj->setErfolgswert(result->Erfolgswert());
				result=neu_obj;
			}
			else result->setZusatz(zus);
		}
	}
	if (!result->Erfolgswert()) // if not already set
		result->setErfolgswert(mbe->FErfolgswert(param.Aben,result));
	result->setLernArt(w);
	return result;
}

H_MBEm LernSchemaElement::lernen(MBEm::wie_erlernt const&w, lernen_param const& param) const
{
	H_MBEm result=lernen(getMBE(),w,param);
	if (!!result) result->setErfolgswert(Erfolgswert());
	return result;
}

#if 0
MBE_T::st_zusatz LernSchemaElement::Auswaehlen(lernen_param const& param) const
{
    return Auswaehlen(getMBE(), param);
}
#endif

#include <iostream>
MBE_T::st_zusatz LernSchemaElement::Auswaehlen(cH_MBE const& mbe, lernen_param const& param)
{
	// Liste zusammenstellen und Auswaehlen aufrufen
	  std::vector<MBE_T::st_zusatz> VG;
	  switch(mbe->ZusatzEnum(param.Aben.getVTyp()))
	   {
	     case MBE_T::ZUeberleben: VG=LernListen::getUeberlebenZusatz(); break;
	     case MBE_T::ZLand:       VG=LernListen::getLandZusatz(); break;
	     case MBE_T::ZSprache:    VG=LernListen::getSprachenZusatz(*mbe,param.Aben,param.nachbarland); break;
	     case MBE_T::ZSchrift:    VG=LernListen::getSchriftenZusatz(*mbe,param.Aben); break;
	     case MBE_T::ZWaffe:      VG=LernListen::getWaffenZusatz(param.Aben.List_Waffen()); break;
	     case MBE_T::ZTabelle:    VG=mbe->VZusatz(); break;
	     default: assert(!"never get here");
	   }

std::cout << "VGsize=" << VG.size() << '\n';
	  if(VG.empty()) return MBE_T::st_zusatz();
	  else if (++VG.begin()==VG.end()) return *VG.begin();
	  else return Auswaehlen(VG,"Welche(r,s) "+mbe->Name()+" soll gelernt werden?");
}

LernSchemaElement::auswahl_ptr LernSchemaElement::current_auswahl_fun;
void* LernSchemaElement::current_auswahl_obj;

void LernSchemaElement::setAuswahl(auswahl_ptr fun, void *obj)
{
	current_auswahl_fun=fun;
	current_auswahl_obj=obj;
}

MBE_T::st_zusatz const& LernSchemaElement::Auswaehlen(std::vector<MBE_T::st_zusatz> const& list, std::string const& title)
{
	return (*current_auswahl_fun)(current_auswahl_obj,list,title);
}

MBE_T::st_zusatz::st_zusatz(cH_MBE const& m, bool e)
	: name(m->Name()), erlaubt(e), region(m->Region()), region_zusatz(m->RegionZusatz()), long_region(m->RegionString()), mbe(m), land(NULL)
{}

MBE_T::st_zusatz::st_zusatz(cH_Land const& m, bool e)
	: name(m->Name()), erlaubt(e), region(), region_zusatz(), long_region(), mbe(NULL),land(m)
{}

void MidgardBasicElement_mutable::loadWieGelernt(Tag const& t)
{
   FOR_EACH_CONST_TAG_OF(i,t,"erlernt")
   {
     gelernt.wann=xml2time(i->getAttr<std::string>("wann"));
     if (i->hasAttr("angeboren")) gelernt.wie=wie_erlernt::angeboren;
     else if (i->hasAttr("Beruf")) gelernt.wie=wie_erlernt::beruf;
     else if (i->hasAttr("Liste"))
     { gelernt.wie=wie_erlernt::lernen;
       gelernt.lernen_detail.lp=i->getAttr<int>("LP");
       gelernt.lernen_detail.liste=xml2liste(i->getAttr<std::string>("Liste"));
     }
     else if (i->hasAttr("Spruchrolle"))
     { gelernt.wie=wie_erlernt::spruchrolle;
       xml2fp(*i,gelernt.spruchrolle_detail);
     }
     else if (i->hasAttr("PP"))
     { gelernt.wie=wie_erlernt::praxis;
       xml2fp(*i,gelernt.praxis_detail.fp);
       gelernt.praxis_detail.praxispunkte=i->getAttr<int>("PP");
     }
     else if (i->hasAttr("GS") || i->hasAttr("AEP") || i->hasAttr("ZEP") || i->hasAttr("KEP"))
     { gelernt.unterweisung_detail.gs=i->getAttr_def<int>("GS");
       xml2fp(*i,gelernt.unterweisung_detail.fp);
       gelernt.wie=wie_erlernt::unterweisung;
       // selbststudium is difficult to recognize
     }
   }
   FOR_EACH_CONST_TAG_OF(i,t,"gesteigert")
   {
	   MBEm::wie_gesteigert wie;
	   wie.wann=xml2time(i->getAttr("wann"));
	   wie.alter_wert=i->getAttr<int>("von");
	   if (i->hasAttr("Beruf")) wie.wie=wie_erlernt::beruf;
	     else if (i->hasAttr("PP"))
	     { wie.wie=wie_erlernt::praxis;
	       xml2fp(*i,wie.praxis_detail.fp);
	       wie.praxis_detail.praxispunkte=i->getAttr<int>("PP");
	     }
	     else if (i->hasAttr("GS") || i->hasAttr("AEP") || i->hasAttr("ZEP") || i->hasAttr("KEP"))
	     { wie.unterweisung_detail.gs=i->getAttr_def<int>("GS");
	       xml2fp(*i,wie.unterweisung_detail.fp);
	       wie.wie=wie_erlernt::unterweisung;
	       // selbststudium is difficult to recognize
	     }
	   gesteigert.push_back(wie);
   }
}

void LernSchemaElement::setLernpunkte(int l)
{
  lernpunkte=l;
  // Überlauf abfangen
  if (lernpunkte<l)
    lernpunkte=15; // Maximalwert
  else if (lernpunkte>l)
    lernpunkte=-16; // Minimalwert
}
