/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef MIDGARDBASICELEMENT_HH
#define MIDGARDBASICELEMENT_HH
#include <Misc/Handles.h>
#include <Misc/CacheStatic.h>
#include <Misc/compiler_ports.h>
#include <string>
#include <vector>
#include <list>
#include <cstring> // memset
//#include "NotFound.h"
//#include <Tag.h>
//#include "xml.h"
//#include "Enums.hh"

class cH_Typen;
class Grundwerte;
class H_Data_beruf;
class cH_MidgardBasicElement;
class SimpleTree;
//class Datenbank;
class MidgardBasicElement_mutable;
class LernSchemaElement;
class Abenteurer;
class Tag;
class H_MidgardBasicElement_mutable;
class cH_Land;

typedef H_MidgardBasicElement_mutable H_MBEm;
typedef cH_MidgardBasicElement cH_MBE;

// dies ist eine erlernbare Fertigkeit, d.h. losgelöst von einem Abenteurer
class MidgardBasicElement : public HandleContent
{
   protected:
      // WARNUNG: Beruf wird nicht verwendet
      struct st_ausnahmen{std::string herkunft;std::string spezies;
                          std::string typ;std::string beruf;std::string stand;
                          std::string standard;
             st_ausnahmen(std::string h,std::string s,std::string t,
                          std::string b,std::string st,std::string sta)
                        :herkunft(h),spezies(s),typ(t),beruf(b),stand(st),
                         standard(sta) {} };
 public:
      enum eZusatz {ZNone,ZTabelle,ZLand,ZWaffe,
                    ZHerkunft,ZSprache,ZSchrift,ZUeberleben};
      struct st_zusatz{std::string name;bool erlaubt; std::string typ;std::string region;
      					std::string region_zusatz; std::string long_region;
      					Handle<const MidgardBasicElement> mbe;
      					Handle<const HandleContent> land;
             st_zusatz() : mbe(NULL),land(NULL) {}
             st_zusatz(std::string const& n,bool e=true)
		: name(n),erlaubt(e), mbe(NULL),land(NULL) {}
             st_zusatz(std::string const& n,std::string const& t,
			std::string const& r,std::string const& rz,
			std::string const& lr,bool e=true)
               : name(n),erlaubt(e),typ(t),region(r),region_zusatz(rz),
			long_region(lr), mbe(NULL),land(NULL) {}
	     st_zusatz(cH_MBE const& m, bool e=true);
	     st_zusatz(cH_Land const& m, bool e=true);
	};
      enum EP_t { Nicht=0, KEP=1, ZEP=2, Beides=KEP|ZEP, EP_t_undefined };
   protected:
//	const Tag *tag; // später weg
      std::string name, region,region_zusatz;
      int kosten;
      mutable int anfangswert;
      eZusatz enum_zusatz;

      std::vector<st_zusatz> Vzusatz;
      std::vector<st_ausnahmen> VAusnahmen;
      bool nsc_only;
      EP_t steigern_mit_EP;
      std::map<std::string,std::string> map_typ;
      std::map<int,int> map_erfolgswert_kosten;

      static std::map<std::string,EP_t> verwendbareEP;
      static std::map<int,std::map<int,int> > waffen_steigern_nach_schwierigkeit;
      static std::map<std::string,std::map<int,int> > sonstige_steigern_kosten;
public:
      static void load_waffen_steigern_nach_schwierigkeit(const Tag &t);
      static void load_steigern_kosten(const Tag &t);

protected:
      void get_map_typ();
      void get_map_typ(const Tag &t);
      void get_Steigern_Kosten_map(const Tag &t);
      const std::map<int,int> &get_Steigern_Kosten_map(); // anhand des Typs bestimmen
      int GrundKosten() const {  return kosten; }

   public:
      MidgardBasicElement(const std::string &n)
            : name(n), kosten(),anfangswert(),enum_zusatz()
              ,nsc_only(),steigern_mit_EP() {}

      enum MBEE {BERUF,FERTIGKEIT,FERTIGKEIT_ANG,WAFFEGRUND,WAFFE,WAFFEBESITZ,
                 ZAUBER,ZAUBERWERK,KIDO,SPRACHE,SCHRIFT,SINN,RESISTENZ_UND_CO} ;

      const std::map<std::string,std::string> &get_MapTyp() const {return map_typ;}

      const std::vector<st_zusatz> &VZusatz() const {return Vzusatz;}
      virtual eZusatz ZusatzEnum(const std::vector<cH_Typen>& Typ) const {return enum_zusatz;}

      bool NSC_only() const {return nsc_only;}
      static void EP_steigern(const std::string &fert,EP_t e);
      static EP_t EP_steigern_load(const Tag &t);
      // initialisierung: steigern wie Fertigkeit ...
      EP_t EP_steigern(const std::string &fert);
      void EP_steigern(EP_t e) { steigern_mit_EP=e; }
      void EP_steigern(const Tag &t) { steigern_mit_EP=EP_steigern_load(t); }

      virtual std::string Name() const {return name;}
      int Anfangswert() const {return anfangswert;}
//      void setAnfangswert(int i) const {anfangswert=i;}
      const std::string &Region() const {return region;}
      const std::string &RegionZusatz() const {return region_zusatz;}
      std::string RegionString() const;
      EP_t Steigern_mit_EP() const {return steigern_mit_EP;}
      virtual enum MBEE What() const=0;
      virtual std::string What_str() const=0; // zum speichern
      virtual std::string Stufe() const {return "";}
      virtual int MaxErfolgswert(const Abenteurer &A) const {return 0;};
      virtual int LernLand() const {return -99;}
      virtual int LernStadt() const {return -99;}
      bool ist_lernbar(const std::vector<cH_Typen>& Typ,const std::map<std::string,std::string>& map_typ) const;
      // komische Funktion: Liste von Strings?
//      bool ist_gelernt(const std::list<std::string>& L) const;
      bool ist_gelernt(Abenteurer const& A) const;

      virtual int FErfolgswert(const Abenteurer &abenteurer,const H_MBEm &mbem) const;
      virtual std::string Voraussetzung() const {return "B U G";}
      virtual bool Voraussetzung(const Abenteurer& A,bool anzeigen=true) const;

      int get_Steigern_Kosten(int erfolgswert) const;

      std::vector<std::string> Standard(const Abenteurer &A) const;
      std::string Standard__(const Abenteurer &A) const;
      double Standard_Faktor(const Abenteurer &A) const ;
      bool Grundfertigkeit(const Abenteurer &A) const;
      bool Standardfertigkeit(const Abenteurer &A) const;
private:
      std::string AusnahmenString(const Grundwerte &Werte,const cH_Typen& Typ,const std::string s) const;
public:
      std::string Standard_Faktor(const Grundwerte &Werte,const std::vector<cH_Typen>& Typ,const std::string s) const;

      int Kosten(const Abenteurer &A) const ;
      bool standard_one_G(const std::vector<std::string>& s) const ;
      bool standard_all_S(const std::vector<std::string>& s) const ;
      virtual bool operator == (const MidgardBasicElement& b) const
         {return What()==b.What() && Name()==b.Name();}
      bool operator < (const MidgardBasicElement& b) const
         {return  What()<b.What() || (What()==b.What()
			&& Name()<b.Name());
         }

      static void saveElementliste(Tag &datei,
      				const std::list<H_MBEm>& b,
                                   const Grundwerte& Werte,
                                   const std::vector<cH_Typen>& Typ);
};

// abbreviation
typedef MidgardBasicElement MBE_T;

class cH_MidgardBasicElement : public Handle<const MBE_T>
{
      typedef CacheStatic<std::string,cH_MidgardBasicElement> cache_t;
      static cache_t cache;
      friend class std::map<std::string,cH_MidgardBasicElement>;
      cH_MidgardBasicElement(){}
   public:
      cH_MidgardBasicElement(const MBE_T *r)
            : Handle<const MBE_T>(r){}
      template <class T>
       cH_MidgardBasicElement(const Handle<T> &r)
       	    : Handle<const MBE_T>(r){}
      template <typename T>
       Handle<const T> getHandle() const
      { return cast_dynamic<const T>(); }
#if 1
   class sort {
      public:
         enum esort {/*LERNPUNKTEPFLICHT,*/NAME,ERFOLGSWERT};
      private:
         esort es;
      public:
         sort(esort _es):es(_es) {}
         bool operator() (cH_MidgardBasicElement const& x,cH_MidgardBasicElement const& y) const;
    };
#endif
};

// dies ist eine erlernte "Fertigkeit" d.h. sie hat Erfolgswert usw.
// umbenennen in ErlerntesElement
class MidgardBasicElement_mutable : public HandleContent
{
public:
	static bool do_not_timestamp;
    // noch nicht genutzt
    struct wie_erlernt
    { enum wie_t { angeboren, lernen, beruf, unterweisung, selbststudium,
          spruchrolle, praxis };
      enum lernen_t { fachkenntnis, allgemeinwissen, ungewoehnlich, separat }; // separat: eigene Liste (Waffen, Zauber)
      wie_t wie:4;
    //  int neuer_wert:6; // [-32..31]
      time_t wann;
//      int dauer; // Tage
      struct fp_t { int aep, zep, kep; int Summe() const { return aep+zep+kep; } };
      struct praxis_t { fp_t fp; unsigned praxispunkte:6; };
      struct lernen_det_t { lernen_t liste; int lp; };
      struct unterweisung_t { fp_t fp; unsigned gs; } ;
      union
      { praxis_t praxis_detail;
        lernen_det_t lernen_detail;
        unterweisung_t unterweisung_detail; // +selbststudium
        fp_t spruchrolle_detail;
      };
      wie_erlernt() { memset(this,0,sizeof *this); }
    };

    struct wie_gesteigert
    {  wie_erlernt::wie_t wie:4;
       int alter_wert:7;
       time_t wann;
//      int dauer; // Tage
      union
      { wie_erlernt::praxis_t praxis_detail;
        wie_erlernt::unterweisung_t unterweisung_detail; // +selbststudium
      };
      wie_gesteigert() { memset(this,0,sizeof *this); }
    };

    friend void MBE_T::saveElementliste(Tag &datei,
				const std::list<H_MBEm>& b,
                           const Grundwerte& Werte,
                           const std::vector<cH_Typen>& Typ);
 private:
      cH_MBE mbe;
      int praxispunkte,erfolgswert;
      MBE_T::st_zusatz zusatz; // Für Zusätze bei Fertigkeiten
                              // (z.B. Abrichten, Sprache, Geheimzeichen...)
                              // und Zauber (Tiersprache)
      wie_erlernt gelernt;
      std::vector<wie_gesteigert> gesteigert;
 public:
      template <class T>
       MidgardBasicElement_mutable(const Handle<T> &_mbe)
         : mbe(_mbe),praxispunkte(),erfolgswert()
           {setErfolgswert(mbe->Anfangswert());}
      // wegen der &* Unsitte ;-)
      MidgardBasicElement_mutable(const MBE_T *_mbe)
         : mbe(_mbe),praxispunkte(),erfolgswert()
           {setErfolgswert(mbe->Anfangswert());}
      const MBE_T *operator->() const
         {return &*mbe;}
      bool operator == (const MidgardBasicElement_mutable& b) const
         {return What()==b.What() && mbe==b.mbe && Zusatz()==b.Zusatz();}
#if 1
      // eigentlich sollte das nicht erforderlich sein ...
      bool operator == (const LernSchemaElement& b) const;
#endif
      bool operator < (const MidgardBasicElement_mutable& b) const
         {return What()<b.What() || (What()==b.What()
                   && (mbe<b.mbe || (mbe==b.mbe
                   && Zusatz()<b.Zusatz())));
         }
      const MBE_T *getMBE() const {return &*mbe;}
      template <typename T>
       Handle<const T> getHandle() const
//      { return mbe.cast_dynamic<const T>(); } // geht erst ab g++ 3.4
      { return Handle<const T>(dynamic_cast<const T*>(&*mbe)); }

      enum MBE_T::MBEE What() const {return mbe->What();}
      std::string What_str() const {return mbe->What_str();}
      int Erfolgswert() const {return erfolgswert;};
      void setErfolgswert(int e) {erfolgswert=e;}
      void addErfolgswert(int e, wie_gesteigert const& w); // wann+alter_wert are filled in
      int Praxispunkte() const {return praxispunkte;};
      void setPraxispunkte(int e)  {praxispunkte=e;}
      void addPraxispunkte(int e)  {praxispunkte+=e;}

      const std::string &Zusatz() const {return zusatz.name;}
      void setZusatz(MBE_T::st_zusatz z) {zusatz=z;}
      void setLernArt(wie_erlernt const&w); // wann is automatically set
      wie_erlernt const& getLernArt() const { return gelernt; }
     wie_gesteigert LetztesSteigern() const
     { assert(!gesteigert.empty()); return *--gesteigert.end(); }
     std::vector<wie_gesteigert> &AlleSteigern()
     { return gesteigert; }

     bool ist_gelernt(const std::list<H_MBEm>& L) const;
     bool ist_gelernt(Abenteurer const& A) const;
     int Steigern(const Abenteurer &A) const;
     int Reduzieren(const Abenteurer &A) const;
     int Verlernen(const Abenteurer &A) const;
     int AnzahlLernpunkte() const { return gelernt.wie!=wie_erlernt::lernen ? 0 : gelernt.lernen_detail.lp; }
     void loadWieGelernt(Tag const&);

      MidgardBasicElement_mutable *dup() const;
// wird die Klasse hier gebraucht?
#if 0
   class sort {
      public:
         enum esort {LERNPUNKTEPFLICHT,NAME,ERFOLGSWERT};
      private:
         esort es;
      public:
         sort(esort _es):es(_es) {}
         bool operator() (MidgardBasicElement_mutable x,MidgardBasicElement_mutable y) const;
    };
#endif
};

typedef MidgardBasicElement_mutable MBEm;
typedef MidgardBasicElement_mutable MBEm_T;

class H_MidgardBasicElement_mutable : public Handle<MBEm>
{
      H_MidgardBasicElement_mutable(){}
   public:
      H_MidgardBasicElement_mutable(MBEm *r)
            : Handle<MBEm>(r){}
      H_MidgardBasicElement_mutable(const cH_MBE  &_mbe)
         : Handle<MBEm>(new MBEm(_mbe))
               {}

      // duplizieren d.h. unabhängiges Element erzeugen
      H_MidgardBasicElement_mutable dup() const;
      static H_MidgardBasicElement_mutable null()
	{ return H_MidgardBasicElement_mutable(); }

#if 1
   class sort {
      public:
         enum esort {/*LERNPUNKTEPFLICHT,*/NAME,ERFOLGSWERT};
      private:
         esort es;
      public:
         sort(esort _es):es(_es) {}
         bool operator() (H_MidgardBasicElement_mutable x,H_MidgardBasicElement_mutable y) const;
    };
#endif
};

void operator|=(MBE_T::EP_t &a, MBE_T::EP_t b);
bool operator&(MBE_T::EP_t a, MBE_T::EP_t b);

class LernSchemaElement : public HandleContent
{public:
     typedef MBE_T::st_zusatz const& (*auswahl_ptr)(void* obj, std::vector<MBE_T::st_zusatz> const& list, std::string const& title);
     struct lernen_param
     {
       const Abenteurer &Aben;
       bool nachbarland;
       lernen_param(Abenteurer const& A, bool n=false) : Aben(A), nachbarland(n) {}
     };

 private:
      cH_MBE mbe;
      int erfolgswert:7,lernpunkte:5;
      bool pflicht:1;
      bool erlaubt:1;
      bool gelernt:1; // gelernte werden auch irgendwann (...) angezeigt
      MBEm_T::wie_erlernt::lernen_t liste;
	// evtl. unnötig
//      std::string lernart; // Fach- Allgemeinwissen, ungew. Fert.
      static auswahl_ptr current_auswahl_fun;
      static void* current_auswahl_obj;

 public:
      template <class T>
       LernSchemaElement(const Handle<T> &_mbe)
         : mbe(_mbe),erfolgswert(),lernpunkte(),pflicht(),erlaubt(),gelernt(), liste(MBEm_T::wie_erlernt::fachkenntnis)
           {setErfolgswert(mbe->Anfangswert());}
      bool operator == (const LernSchemaElement& b) const
         {return mbe==b.mbe && erfolgswert==b.erfolgswert;}
#if 0
      // eigentlich sollte das nicht erforderlich sein ...
      bool operator == (const MBEm& b) const
         {return mbe==b.getMBE() && erfolgswert==b.Erfolgswert(); }
#endif
      bool operator < (LernSchemaElement const& b) const
         {return mbe<b.mbe || (mbe==b.mbe
                   && erfolgswert<b.erfolgswert);
         }
      const MBE_T *getMBE() const {return &*mbe;}
      template <typename T>
       Handle<const T> getHandle() const
//      { return mbe.cast_dynamic<const T>(); } // geht erst ab g++ 3.4
      { return Handle<const T>(dynamic_cast<const T*>(&*mbe)); }

      enum MBE_T::MBEE What() const {return mbe->What();}
      std::string What_str() const {return mbe->What_str();}
      int Lernpunkte() const {return lernpunkte;};
      void setLernpunkte(int l);
      int Erfolgswert() const {return erfolgswert;};
      void setErfolgswert(int e) {erfolgswert=e;}

      // convenience function
      MBE_T::st_zusatz Auswaehlen(lernen_param const& param) const;
      static MBE_T::st_zusatz Auswaehlen(cH_MBE const& mbe, lernen_param const& param);
      static MBE_T::st_zusatz const& Auswaehlen(std::vector<MBE_T::st_zusatz> const&, std::string const& title);

      __deprecated MBE_T const*operator->() const
         {return &*mbe;}

#if 0
      const std::string &Zusatz() const {return zusatz.name;}
      void setZusatz(MBE_T::st_zusatz z) {zusatz=z;}
#endif

//     const std::string &LernArt() const {return lernart;}
//     void setLernArt(const std::string &z)  {lernart=z;}
     MBEm_T::wie_erlernt::lernen_t getListe() const { return liste; }
     void setListe(MBEm_T::wie_erlernt::lernen_t l) { liste=l; }

     bool Erlaubt() const {return erlaubt;}
     void setErlaubt(bool e) {erlaubt=e;}
     bool Pflicht() const {return pflicht;}
     void setPflicht(bool p) {pflicht=p;}
     std::string Pflicht_str() const;
     bool Gelernt() const {return gelernt;}
	// CPE: this const cast is a hack ...
     void setGelernt(bool e) const {const_cast<LernSchemaElement*>(this)->gelernt=e;}

     bool ist_gelernt(Abenteurer const& A) const;
     bool ist_gelernt(const std::list<H_MBEm>& L) const;
     bool ist_gelernt(const std::list<Handle<LernSchemaElement const> >& L) const;
     static void setAuswahl(auswahl_ptr fun, void* obj);
     // der Abenteurer wird zum Spezifizieren von Zusätzen benötigt
     H_MBEm lernen(MBEm::wie_erlernt const& w, lernen_param const& param) const;
     // helper function
     static H_MBEm lernen(cH_MBE const& was, MBEm::wie_erlernt const&w, lernen_param const& param);

//      MBEm *dup() const;
   class sort {
      public:
         enum esort {LERNPUNKTEPFLICHT,NAME,ERFOLGSWERT};
      private:
         esort es;
      public:
         sort(esort _es):es(_es) {}
         bool operator() (Handle<LernSchemaElement const> const& x,Handle<LernSchemaElement const> const& y) const;
    };
};

typedef LernSchemaElement LSE_T;
//typedef Handle<LernSchemaElement const> hLSE; // old deprecated typing
typedef Handle<LernSchemaElement const> H_LSE; // just for convenience

      // eigentlich sollte das nicht erforderlich sein ...
inline bool MBEm::operator == (const LernSchemaElement& b) const
{ return mbe==b.getMBE() && erfolgswert==b.Erfolgswert(); }

#endif
