/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Zufall.hh"
#include "Sprache.hh"
#include "Schrift.hh"
#include "Land.hh"
#include "WaffeGrund.hh"
#include "class_lernpunkte.hh"
#include <Misc/relops.h>
//#include <Misc/relops.h>
#include "Ausgabe.hh"
#include "Random.hh"
#include "LernListen.hh"

enum Zufall::eFAUWZ &operator++(enum Zufall::eFAUWZ &s)
{  ++(int&)s;
   return s;
}

void Zufall::Lernschema()
{
  Lernpunkte_wuerfeln(Aben);
  st_LL FAUWZ_Listen=getLernlisten();

  if(Aben.Typ1()->getLernpflichtSchrift() || Aben.Typ2()->getLernpflichtSchrift())
   {
     int i=Random::integer(1,2);
     if     (i==1) Aben.getLernpunkte().getLernpunkte().set_schreiben_pflicht_allg(true);
     else if(i==2) Aben.getLernpunkte().getLernpunkte().set_schreiben_pflicht_fach(true);
   }

  for(eFAUWZ i=eWaffen;i<eMAX;++i)
     Lernpunkte_verteilen(i,FAUWZ_Listen);

  setSpezialwaffe();
  setSpezialgebiet();
}

void Zufall::setSpezialwaffe()
{
	Aben.SpezialwaffeWaehlen();
}

void Zufall::setSpezialgebiet()
{
	Aben.SpezialgebietWaehlen();
}

// ???
std::vector<H_LSE> List_to_Vector(std::list<H_LSE> L,const Abenteurer& Aben,int lp)
{
  std::vector<H_LSE> V;
  for(std::list<H_LSE>::const_iterator i=L.begin();i!=L.end();++i)
   {
     // Vorraussetzungen?
     if(((*(*i)).What()==MBE_T::FERTIGKEIT || (*(*i)).What()==MBE_T::WAFFE)
          && !(*(*i)).getMBE()->Voraussetzung(Aben,false))  continue;
     if((*i)->Lernpunkte()<=lp)   V.push_back(*i);
   }
  return V;
}

static std::vector<H_MBEm> List_to_Vector(std::list<H_MBEm> L,const Abenteurer& Aben)
{
  std::vector<H_MBEm> V;
  for(std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
   {
     // Vorraussetzungen?
     if(((*(*i)).What()==MBE_T::FERTIGKEIT || (*(*i)).What()==MBE_T::WAFFE)
          && !(*(*i)).getMBE()->Voraussetzung(Aben,false))  continue;
     V.push_back(*i);
   }
  return V;
}

void Zufall:: Lernpunkte_verteilen(const eFAUWZ was, const st_LL &Listen)
{
 std::list<H_LSE> L;
 int lp=0;
 bool mutter_12=false,mutter_9=false;
 switch (was) {
   case eWaffen: lp=Aben.getLernpunkte().getLernpunkte().Waffen(); L=Listen.Waff; break;
   case eZauber: lp=Aben.getLernpunkte().getLernpunkte().Zauber(); L=Listen.Zaub; break;
   case eFach:   lp=Aben.getLernpunkte().getLernpunkte().Fach();   L=Listen.Fach; mutter_12=Aben.getLernpunkte().getLernpunkte().get_schreiben_pflicht_fach(); break;
   case eAllg:   lp=Aben.getLernpunkte().getLernpunkte().Allgemein(); L=Listen.Allg; mutter_9=Aben.getLernpunkte().getLernpunkte().get_schreiben_pflicht_allg();break;
   case eUnge:   lp=Aben.getLernpunkte().getLernpunkte().Unge();   L=Listen.Unge; break;
   default: return;
  }
  std::list<H_MBEm> List_gelerntes;
reloop:
  L.sort(LernSchemaElement::sort(LernSchemaElement::sort::LERNPUNKTEPFLICHT));
  std::vector<H_LSE> V=List_to_Vector(L,Aben,lp); // Lernpunkte und Vorraussetzungen
  while(lp>0)
   {
     if(V.begin()==V.end()) break;
     if(knows_everything(List_gelerntes,L)) return;
     if ((*V.begin())->Lernpunkte()>lp) break; // all too expensive
     int i;
     std::vector<H_LSE>::const_iterator ci;
     try{
     if(V[0]->Lernpunkte()==0) i=0;  // Fertigkeiten mit '0' Lernpunkten zuerst
     else if(V[0]->Pflicht()) i=0;   // Pflichtfertigkeiten zuerst
     else i=Random::integer(0,V.size()-1);
     }catch(std::exception &e){
        Ausgabe(Ausgabe::Error,std::string("Lernpunkte_verteilen: Exception ")+e.what());
        i=Random::integer(0,V.size()-1);}

     if(i!=-1)  ci=V.begin()+i;
     if ((*ci)->Lernpunkte()>lp) goto reloop;
	 
	 H_MBEm M=Aben.Erlernen(*ci);
	 if (!!M)
	 {
	       lp-=M->AnzahlLernpunkte();
	       if (!M->AnzahlLernpunkte() || M->getMBE()->ZusatzEnum(Aben.getVTyp())==MBE_T::ZNone)
	         L.remove(*ci);
	       goto reloop;
	 }
   }  
}

bool operator==(H_MBEm const& a, H_LSE const& b)
{  return a->What()==b->What() && a->getMBE()==b->getMBE();
}

// needed for C++ compliance?
bool operator==(H_MBEm const& a, H_MBEm const& b)
{  return *a==*b;
}

bool Zufall::knows_everything(const std::list<H_MBEm> &List_gelerntes,const std::list<H_LSE> &L)
{
  
  for(std::list<H_LSE>::const_iterator i=L.begin();i!=L.end();++i)
   {
     std::list<H_MBEm>::const_iterator j;
     j=find(List_gelerntes.begin(),List_gelerntes.end(),*i);
     if(j==List_gelerntes.end()) return false;
   }
  return true;
}




Zufall::st_LL Zufall::getLernlisten()
{
   
  st_LL FAUWZ_Listen=st_LL( LernListen::getMBEm(Aben,LernListen::lFach),
                            LernListen::getMBEm(Aben,LernListen::lAllg),
                            LernListen::getMBEm(Aben,LernListen::lUnge),
                            LernListen::getMBEm(Aben,LernListen::lWaff),
                            LernListen::getMBEm(Aben,LernListen::lZaub));

  return FAUWZ_Listen;
}

#if 0
H_MBEm Zufall::getZusatz(MBE_T::eZusatz was,H_MBEm& MBE,bool nachbarland) const
{
//cout << "Zusatz für "<<MBE->Name()<<'\n';
  std::vector<MBE_T::st_zusatz> VG;
  switch(was)
   {
//     case MBE_T::ZHerkunft:
     case MBE_T::ZUeberleben: VG=LernListen::getUeberlebenZusatz();break;
     case MBE_T::ZLand:       VG=LernListen::getLandZusatz(); break;
     case MBE_T::ZSprache:    VG=LernListen::getSprachenZusatz(*MBE->getMBE(),Aben,nachbarland); break;
     case MBE_T::ZSchrift:    VG=LernListen::getSchriftenZusatz(*MBE->getMBE(),Aben); break;
     case MBE_T::ZWaffe:      VG=LernListen::getWaffenZusatz(Aben.List_Waffen()); break;
     case MBE_T::ZTabelle:    VG=(*MBE)->VZusatz(); break;
     default: assert(!"never get here");
   }
  std::vector<MBE_T::st_zusatz> &V=VG;
/*
  for (std::vector<MBE_T::st_zusatz>::const_iterator i=VG.begin();i!=VG.end();++i)
   {
     if(???char->region_check(i->region)) V.push_back(*i);
   }
*/
  if(V.empty()) return MBE;
  int i=Random::integer(0,V.size()-1);

  MBE->setZusatz(V[i]);

  H_MBEm Mtmp=MBE;

  if(was==MBE_T::ZSprache)
       Mtmp=H_MBEm(cH_Sprache(V[i].name));
  else if(was==MBE_T::ZSchrift)
       Mtmp=H_MBEm(cH_Schrift(V[i].name));

  if(MBE != Mtmp)
   { // was soll das hier? kann nur passieren, wenn oben Mtmp geändert wurde
     Mtmp->setErfolgswert(MBE->Erfolgswert());
     Mtmp->setLernArt(MBE->getLernArt());
     MBE=Mtmp;
   }
  return MBE;
}
#endif

std::vector<H_WaffeBesitz> List_to_Vector(const std::list<H_WaffeBesitz>& L)
{
  std::vector<H_WaffeBesitz> V;
  for(std::list<H_WaffeBesitz>::const_iterator i=L.begin();i!=L.end();++i)
   {
     V.push_back(*i);
   }
  return V;
}

void Zufall::setWaffenBesitz()
{
  std::list<H_WaffeBesitz> L=LernListen::getWaffenBesitz(Aben);
  WaffeBesitzLernen wbl=WaffenBesitz_wuerfeln(Aben,Random::W100());

  // CP: Sehr ungewöhnliche Art, Schleifen zu schreiben, fast unverständlich ...
reloop:
  if(wbl.AWaffe() == 0 && wbl.EWaffe() == 0) return;
  std::vector<H_WaffeBesitz> V=List_to_Vector(L);
  while (!L.empty())
  //for(std::vector<H_WaffeBesitz>::const_iterator i=V.begin();i!=V.end();++i)
   {
     int i=Random::integer(0,V.size()-1);
     H_WaffeBesitz WB=V[i];
     std::string art=cH_Waffe(WB->getMBE())->Art2();     
     if( (art=="E" || art=="W" || art=="V") && wbl.EWaffe()>0)
      {
        wbl.add_EWaffe(-1);
        if(wbl.getMagisch())
          {
            WB->set_av_Bonus(1);
            wbl.setMagisch(false);
          }
      }
     else if(wbl.AWaffe()>0)  wbl.add_AWaffe(-1)  ;  
     else continue;
     Aben.List_Waffen_besitz().push_back(WB);
     L.remove(WB);
     goto reloop;
   }
}

void Zufall::setBeruf()
{
  std::list<H_MBEm> L=LernListen::getBeruf(Aben);
  std::vector<H_MBEm> V=List_to_Vector(L,Aben);
  if(V.empty()) return;
  int i=Random::integer(0,V.size()-1);
  Aben.List_Beruf().clear();
  Aben.List_Beruf().push_back(V[i]);
  
  BerufsKategorie BKat;
  BKat.wuerfeln(Random::W100());
  std::vector<Beruf::st_vorteil> F=LernListen::getBerufsVorteil(V[i],BKat,Aben);
  if(F.empty()) return;

  while(true)
   {
     i=Random::integer(0,F.size()-1);
     Beruf::Berufsfertigkeit(Aben,F[i]);
     if(F[i].kat==3 || F[i].kat==4) break;
     if(!BKat.kat_IV) break;
     else BKat.kat_IV=false;
   }
}
