/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2004-2007 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "class_SimpleTree.hh"
#include <Misc/EntryValueEmptyInt.h>
#include <Misc/EntryValueIntString.h>
#include <Misc/EntryValueIntGermanString.h>
#include <MidgardBasicTree.hh>
#include <Sprache.hh>
#include <Fertigkeiten.hh>
#include <Fertigkeiten_angeboren.hh>
#include <Schrift.hh>
#include "Waffe.hh"
#include "KiDo.hh"
#include "Zauber.hh"
#include "Zauberwerk.hh"
#include "midgard_CG.hh"
#include <Misc/itos.h>
#include <Misc/create_parse.h>

cH_EntryValue Data_SimpleTree::retEV(const WEV &what) const
{
  switch (what) {
      case FName: { std::string result=ungelernt->Name();
      	if (!!MBE) result+=" "+(*MBE).Zusatz();
      	return cH_EntryValueIntGermanString(result);
      }
      case FErfolgswert: if (!!lse) return cH_EntryValueEmptyInt(lse->Erfolgswert());
          if (!!MBE && ungelernt->What()!=MBE_T::FERTIGKEIT)
              return cH_EntryValueEmptyInt((*MBE).Erfolgswert());
          if (!!MBE)
              return cH_EntryValueEmptyInt(ungelernt->FErfolgswert(abenteurer->getAbenteurer(),MBE));
          return cH_EntryValueEmptyInt(ungelernt->Anfangswert()); 
      case FWurf: {cH_Fertigkeit_angeborene F(ungelernt); return cH_EntryValueIntString(itos(F->Min())+"-"+itos(F->Max()));}
      case FWhat: return cH_EntryValueIntString(ungelernt->What_str());
      case FLernpunkte: return cH_EntryValueEmptyInt((*MBE).AnzahlLernpunkte());
      case FLernpunkte0: 
          if (!lse) return cH_EntryValue();
          return cH_EntryValueIntString((*lse).Lernpunkte());
      case FLernart: 
          if (!lse) return cH_EntryValue();
          switch((*lse).getListe())
          { case MBEm_T::wie_erlernt::fachkenntnis: return cH_EntryValueIntString("Fachkenntnis");
            case MBEm_T::wie_erlernt::allgemeinwissen: return cH_EntryValueIntString("Allgemeinwissen");
            case MBEm_T::wie_erlernt::ungewoehnlich: return cH_EntryValueIntString("ungewöhnlich");
            case MBEm_T::wie_erlernt::separat: return cH_EntryValueIntString("(separat)");
            default: return cH_EntryValueIntString(int((*lse).getListe()));
          }
      case FPflicht: 
          if (!lse) return cH_EntryValue();
          return cH_EntryValueIntString((*lse).Pflicht_str());
      case FGrundkenntnis: return cH_EntryValueIntString(cH_Waffe(ungelernt)->Grundkenntnis());
      case FErfolgswerBonus: { int AB = cH_Fertigkeit(ungelernt)->AttributBonus(abenteurer->getAbenteurer());
                               const int EW=!!MBE ? ungelernt->FErfolgswert(abenteurer->getAbenteurer(),MBE)
                            		   : !!lse ? lse->Erfolgswert() : 0;
                               if(!AB) return cH_EntryValueEmptyInt(EW);
                               else { std::string s=itos(EW-AB)+"+"+itos(AB);
                                      return cH_EntryValueIntString(s); } }
      case FZauberAP: return cH_EntryValueIntString(cH_Zauber(ungelernt)->Ap());
      case FAttribut: return cH_EntryValueIntString(cH_Fertigkeit(ungelernt)->Attribut());
      case FVoraussetung: return cH_EntryValueIntString(ungelernt->Voraussetzung());
      case FSchwierigkeit: return cH_EntryValueIntString(cH_Waffe(ungelernt)->Schwierigkeit_str());
      case FKosten: return cH_EntryValueEmptyInt(ungelernt->Kosten(abenteurer->getAbenteurer()));
      case FStandard: return cH_EntryValueIntString(ungelernt->Standard__(abenteurer->getAbenteurer()));
      case FGelernt: if (!!lse && (*lse).Gelernt()) return cH_EntryValueIntString("*"); 
                     else return cH_EntryValue();
      case FPraxispunkt: return cH_EntryValueEmptyInt((*MBE).Praxispunkte());
      case FSteigern: return cH_EntryValueEmptyInt((*MBE).Steigern(abenteurer->getAbenteurer()));
      case FReduzieren: return cH_EntryValueEmptyInt((*MBE).Reduzieren(abenteurer->getAbenteurer()));
      case FVerlernen: return cH_EntryValueEmptyInt((*MBE).Verlernen(abenteurer->getAbenteurer()));
      case FRegion: return cH_EntryValueIntString(ungelernt->RegionString());
      case FUrsprung: return cH_EntryValueIntString(cH_Zauber(ungelernt)->Ursprung());
      case FArtderSchrift: return cH_EntryValueIntString(cH_Schrift(ungelernt)->Art_der_Schrift());
   }
 return cH_EntryValue();
}

cH_EntryValue Data_SimpleTree::Value(guint seqnr,gpointer gp) const
{ 
      MidgardBasicTree::variante Variante = 
               MidgardBasicTree::variante(reinterpret_cast<long>(gp));
      if (Variante==MidgardBasicTree::ANGEBFERT)
       switch((Spalten_ANGEBFERT)seqnr) {
         case WURFaf  : return retEV(FWurf);
         case NAMEaf  : return retEV(FName);
         case WERTaf  : return retEV(FErfolgswert);
        }
      else if (Variante==MidgardBasicTree::GELERNTES)
       switch((Spalten_GELERNTES)seqnr) {
         case ARTgg  : return retEV(FWhat);
         case NAMEgg : return retEV(FName);
         case WERTgg : return retEV(FErfolgswert);
         case LERNPUNKTEgg : return retEV(FLernpunkte); 
         case LERNARTgg : return retEV(FLernart); }
      else if (Variante==MidgardBasicTree::LERNSCHEMA)
       switch((Spalten_LERNSCHEMA)seqnr) {
         case LERNPUNKTEg : return retEV(FLernpunkte0);
         case PFLICHTg : { if ( ungelernt->What()==MBE_T::FERTIGKEIT)
                                return retEV(FPflicht); 
                           if ( ungelernt->What()==MBE_T::WAFFE)
                                return retEV(FGrundkenntnis);}
         case NAMEg : return retEV(FName);
         case WERTg : 
             if(ungelernt->What()==MBE_T::FERTIGKEIT) return retEV(FErfolgswerBonus);
             else if(ungelernt->What()==MBE_T::WAFFE) return retEV(FErfolgswert);
             else if(ungelernt->What()==MBE_T::ZAUBER)return retEV(FZauberAP);
         case EIGENSCHAFTg : 
             if ( ungelernt->What()==MBE_T::FERTIGKEIT) return retEV(FAttribut);
         case VORAUSSETZUNGg : return retEV(FVoraussetung);
         case KOSTENg : 
            if ( ungelernt->What()==MBE_T::WAFFE) return retEV(FSchwierigkeit);
            return  retEV(FKosten);
         case ARTg : return retEV(FStandard);
         case GELERNTg : return retEV(FGelernt);
        }
      else if (Variante==MidgardBasicTree::LONG_ALT_ || 
               Variante==MidgardBasicTree::SPRACHEN_ALT ||
               Variante==MidgardBasicTree::WAFFE_ALT )
       switch((Spalten_LONG_ALT)seqnr) {
         case NAMEa :  return retEV(FName);
         case WERTa :  return retEV(FErfolgswert);
         case PPa : return retEV(FPraxispunkt);
         case STANDARDa : return retEV(FStandard);
         case STEIGERN : return retEV(FSteigern);
         case REDUZIEREN : return retEV(FReduzieren);
         case VERLERNEN : return retEV(FVerlernen);
         case REGIONa : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::LONG_NEU)
       switch ((Spalten_LONG_NEU)seqnr) {
         case NAMEn : return retEV(FName);
         case WERTn : return retEV(FErfolgswert);
         case LERNKOSTEN : return retEV(FKosten);
         case ART : return retEV(FStandard);
         case VORAUSSETZUNGEN : return retEV(FVoraussetung);
         case REGIONn : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::LONG_NEU_WAFFE)
       switch ((Spalten_LONG_NEU_WAFFE)seqnr) {
         case NAMEnw : return retEV(FName);
         case ARTw : return retEV(FStandard);
         case VORAUSSETZUNGENw : return retEV(FVoraussetung);
         case REGIONnw : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::SPRACHE_NEU)
       switch ((Spalten_SPRACHE_NEU)seqnr) {
         case NAMEsn    : return retEV(FName);
         case SCHRIFTsn : return cH_EntryValueIntString(cH_Sprache(ungelernt)->Schriften()); 
         case LERNKOSTENsn : return retEV(FKosten);
         case REGIONsn : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::SCHRIFT_ALT)
       switch((Spalten_SCHRIFT_ALT)seqnr) {
         case NAMEsa : return retEV(FName);
         case ARTsa : return retEV(FArtderSchrift);
         case WERTsa : return retEV(FErfolgswert);
         case PPsa : return retEV(FPraxispunkt);
         case STANDARDsa : return retEV(FStandard);
         case STEIGERNs : return retEV(FSteigern);
         case REDUZIERENs :  return retEV(FReduzieren);
         case VERLERNENs :  return retEV(FVerlernen);
         case REGIONsa : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::SCHRIFT_NEU)
       switch ((Spalten_SCHRIFT_NEU)seqnr) {
         case NAMErsn    : return retEV(FName);
         case ARTrsn     : return retEV(FArtderSchrift);
         case KOSTENn_SC : return retEV(FKosten);
         case REGIONrsn  : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::KIDO_ALT || Variante==MidgardBasicTree::KIDO_NEU)
       {cH_KiDo k(ungelernt);
        switch ((Spalten_KIDO)seqnr) {
         case HOHOa_K    : return cH_EntryValueIntString(k->HoHo());
         case NAMEa_K    : return cH_EntryValueIntString(k->Deutsch());
         case STUFEa_K   : return cH_EntryValueIntString(k->Stufe());
         case APa_K      : return cH_EntryValueEmptyInt(k->Ap());
         case KOSTENa_K  : return retEV(FKosten);
         case STILa_K    : return cH_EntryValueIntString(k->Stil()); 
        }}
      else if (Variante==MidgardBasicTree::WAFFEGRUND_NEU || Variante==MidgardBasicTree::WAFFEGRUND_ALT)
       switch((Spalten_WAFFEGRUND)seqnr) {
         case NAMEa_G : return retEV(FName);
         case STANDARDa_G : return retEV(FStandard);
         case KOSTEN_G : return retEV(FKosten);
         case REGION_G : return retEV(FRegion);
        }
      else if (Variante==MidgardBasicTree::ZAUBER_NEU || Variante==MidgardBasicTree::ZAUBER_ALT )
       { cH_Zauber z(ungelernt);
       switch ((Spalten_ZAUBER)seqnr) {
         case STUFEn_Z : return cH_EntryValueIntString(z->Stufe());
         case NAMEn_Z : return retEV(FName);
         case URSPRUNGn_Z : return cH_EntryValueIntString(z->Ursprung());
         case KOSTENn_Z : return retEV(FKosten);
         case STANDARDn_Z : return  retEV(FStandard);
         case REGIONn_Z : return retEV(FRegion);
        }}
      else if (Variante==MidgardBasicTree::ZAUBERWERK_NEU || Variante==MidgardBasicTree::ZAUBERWERK_ALT)
       {cH_Zauberwerk z(ungelernt);
        switch ((Spalten_ZAUBERWERK)seqnr) {
         case STUFEn_ZW : return cH_EntryValueIntString(z->Stufe());
         case NAMEn_ZW : return retEV(FName);
         case ARTn_ZW : return cH_EntryValueIntString(z->Art());    
         case KOSTENn_ZW : return retEV(FKosten);
         case PREISn_ZW : return cH_EntryValueIntString(z->Preis()); 
         case ZEITAUFWANDn_ZW : return cH_EntryValueIntString(z->Zeitaufwand());
         case REGIONn_ZW : return retEV(FRegion);
        }}
      else if (Variante==MidgardBasicTree::WAFFE_LERNSCHEMA)
       {
        cH_Waffe w(ungelernt);
        switch ((Spalten_WAFFE_LERNSCHEMA)seqnr) {
         case NAME_WL    : return retEV(FName);
         case GRUND_WL   : return retEV(FGrundkenntnis);
         case ART_WL     : return cH_EntryValueIntString(w->Art2());
         case SCHADEN_WL : return cH_EntryValueIntString(w->Schaden(w->Name())+"+"+itos(w->Schaden_Bonus(w->Name())));
        }  
       }
      return cH_EntryValue();
}

void Data_SimpleTree::redisplay(SimpleTree *tree) const
{
  tree->redisplay(this,Data_SimpleTree::WERTa);
  tree->redisplay(this,Data_SimpleTree::PPa);
  tree->redisplay(this,Data_SimpleTree::STEIGERN);
  tree->redisplay(this,Data_SimpleTree::REDUZIEREN);
  tree->redisplay(this,Data_SimpleTree::VERLERNEN);
}

bool Data_SimpleTree::changeValue(guint idx,gpointer gp,const Glib::ustring &newval)
{ MidgardBasicTree::variante Variante = 
               MidgardBasicTree::variante(reinterpret_cast<long>(gp));
	if (!MBE) return false;
  try
   { if ((Variante==MidgardBasicTree::LONG_ALT_ || 
               Variante==MidgardBasicTree::SPRACHEN_ALT ||
               Variante==MidgardBasicTree::WAFFE_ALT) 
        && idx==Data_SimpleTree::PPa 
        && ManuProC::parse<int>(newval)!=(*MBE).Praxispunkte())
     {  MBE->setPraxispunkte(ManuProC::parse<int>(newval));
        return true;
     }
     if (Variante==MidgardBasicTree::SCHRIFT_ALT
        && idx==Data_SimpleTree::PPsa 
        && ManuProC::parse<int>(newval)!=(*MBE).Praxispunkte())
     {  MBE->setPraxispunkte(ManuProC::parse<int>(newval));
        return true;
     }
   } catch(...) {} // parse can throw
   return false;
}
