/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2004-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "table_steigern.hh"
#include "midgard_CG.hh"
#include "class_SimpleTree.hh"
#include "Fertigkeiten.hh"
#include "Sprache.hh"
#include "Zauber.hh"
#include <Misc/itos.h>
#include <libmagus/Ausgabe.hh>
#include <libmagus/Datenbank.hh>

enum { Button_Steigern, Button_Verlernen, Button_PP_eingeben };

bool table_steigern::MidgardBasicElement_leaf_alt(const cH_RowDataBase &d)
{
 const Data_SimpleTree *dt=dynamic_cast<const Data_SimpleTree*>(&*d);
 H_MBEm &MBE = const_cast<H_MBEm&>(dt->getMBE());

 ////////////////////////////////////////////////////////////////////////
 AbenteurerAuswahl::LocalUndoRememberer undo(hauptfenster->getChar().actualIterator());

 if((*MBE).What()==MBE_T::FERTIGKEIT 
    && (*MBE)->Name()=="KiDo" 
    && kido_steigern_check(MBE->Erfolgswert())) 
   return false;

 //////////////////////////////////////////////////////////////////////////
// Abenteurer::e_wie_steigern wie=get_wie_steigern();
// Abenteurer::st_bool_steigern bool_steigern=get_bool_steigern();
 
 if (!hauptfenster->getAben().reduzieren && MBE->Steigern(hauptfenster->getAben()))
    { bool result=false;
      getKnownTree((*MBE).What())->getModel().about_to_change(d);
      if (hauptfenster->getAben().Steigern(MBE)) result=true;
      getKnownTree((*MBE).What())->getModel().has_changed(d);
      if (result) 
      { refresh_gesteigert();
        undo.finish((*MBE)->What_str()+" "+(*MBE)->Name()+" auf "+itos(MBE->Erfolgswert())+" gesteigert");
      }
      return result;
    }
 else if (hauptfenster->getAben().reduzieren)
//#warning mit neuen Funtionen realisieren
    { bool result=false;
      getKnownTree((*MBE).What())->getModel().remove_line(d);
      bool verlernt=false;
      if (hauptfenster->getAben().ReduzierenVerlernen(MBE,verlernt)) 
        result=true;
      if (result && verlernt) 
      { /*std::list<H_MBEm> *MyList=&hauptfenster->getAben().getList((*MBE).What()),
               *MyList_neu=&getLearnList((*MBE).What());
        Abenteurer::move_element(*MyList,*MyList_neu,MBE);*/
//        getLearnTree((*MBE).What())->getModel().append_line(d);
//#warning überdenken!
      }
      else getKnownTree((*MBE).What())->getModel().append_line(d);
      
      if (result) 
      { refresh_gesteigert();
        undo.finish((*MBE)->What_str()+" "+(*MBE)->Name()+" auf "+itos(MBE->Erfolgswert())+" verlernt");
      }
      return result;
    }
 return false;
}

void table_steigern::lernen_von_spruchrolle_fragen(const int bonus)
{
 Handle<const Data_SimpleTree> dt=neue_zauber_tree->getSelectedRowDataBase().cast_dynamic<const Data_SimpleTree>();
 cH_MBE MBE = dt->get_ungelernt(); 
 neu_lernen(MBE,bonus);
}

void table_steigern::MidgardBasicElement_leaf_neu(const cH_RowDataBase &d)
{
 Handle<const Data_SimpleTree> dt=d.cast_dynamic<const Data_SimpleTree>();
 cH_MBE MBE = dt->get_ungelernt();
 H_MBEm learned = neu_lernen(MBE);
 if (!!learned)
 { if (!(MBE)->ZusatzEnum(hauptfenster->getAben().getVTyp()))
     getLearnTree((MBE)->What())->getModel().remove_line(d);
// dies ist das das richtige Element auch bei Zusaetzen (s.u.)
   getKnownTree((MBE)->What())->getModel().append_line(new Data_SimpleTree(learned,hauptfenster->getChar().actualIterator()));
   refresh_gesteigert();
 }
}

// dies sollte in VAbenteurer::item
H_MBEm table_steigern::neu_lernen(cH_MBE const& MBE,const int bonus)
{hauptfenster->getChar()->begin_undo();
 H_MBEm result=hauptfenster->getAben().Erlernen(MBE,bonus);
 if(!result) 
   hauptfenster->getChar()->cancel_undo();
 else
   hauptfenster->getChar()->name_undo((MBE)->What_str()+" "+(MBE)->Name()+" gelernt");
 return result;
}

std::list<cH_MBE> &table_steigern::getLearnList(MBE_T::MBEE was)
{ switch (was)
  { case MBE_T::FERTIGKEIT: return list_Fertigkeit_neu;
    case MBE_T::WAFFE: return list_Waffen_neu;
    case MBE_T::WAFFEGRUND: return list_WaffenGrund_neu;
    case MBE_T::ZAUBER: return list_Zauber_neu;
    case MBE_T::ZAUBERWERK: return list_Zauberwerk_neu;
    case MBE_T::KIDO: return list_Kido_neu;
    case MBE_T::SPRACHE: return list_Sprache_neu;
    case MBE_T::SCHRIFT: return list_Schrift_neu;
    default: assert(!"getLearnList: invalid arg");
  }
  abort();
}

MidgardBasicTree *table_steigern::getKnownTree(MBE_T::MBEE was)
{ switch (was)
  { case MBE_T::FERTIGKEIT: return alte_fert_tree;
    case MBE_T::WAFFEGRUND: return alte_grund_tree;
    case MBE_T::WAFFE: return alte_waffen_tree;
    case MBE_T::ZAUBER: return alte_zauber_tree;
    case MBE_T::ZAUBERWERK: return alte_zaubermittel_tree;
    case MBE_T::KIDO: return alte_kido_tree;
    case MBE_T::SPRACHE: return alte_sprache_tree;
    case MBE_T::SCHRIFT: return alte_schrift_tree;
    default: assert(!"getKnownTree: invalid arg");
    // WAFFEBESITZ: waffenbesitz_alt_tree
  }
  abort();
}

MidgardBasicTree *table_steigern::getLearnTree(MBE_T::MBEE was)
{ switch (was)
  { case MBE_T::FERTIGKEIT: return neue_fert_tree;
    case MBE_T::WAFFEGRUND: return neue_grund_tree;
    case MBE_T::WAFFE: return neue_waffen_tree;
    case MBE_T::ZAUBER: return neue_zauber_tree;
    case MBE_T::ZAUBERWERK: return neue_zaubermittel_tree;
    case MBE_T::KIDO: return neue_kido_tree;
    case MBE_T::SPRACHE: return neue_sprache_tree;
    case MBE_T::SCHRIFT: return neue_schrift_tree;
    default: assert(!"getLearnTree: invalid arg");
    // WAFFEBESITZ: waffenbesitz_neu_tree
  }
  abort();
}
